package msg_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/msg/internal/model"
	"github.com/ThreeDotsLabs/watermill"
	"github.com/ThreeDotsLabs/watermill/message/subscriber"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"

	"gitea.dwysokinski.me/twhelp/core/internal/msg"
	"gitea.dwysokinski.me/twhelp/core/internal/msg/internal/mock"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestEnnoblementConsumer_refresh(t *testing.T) {
	t.Parallel()

	marshaler := msg.GobMarshaler{}
	pubSub := newPubSub(t)

	ennoblementSvc := &mock.FakeEnnoblementService{}
	ennoblementSvc.RefreshReturns(nil)

	runRouter(t, msg.NewEnnoblementConsumer(marshaler, pubSub, pubSub, watermill.NopLogger{}, ennoblementSvc))

	msgs, err := pubSub.Subscribe(context.Background(), "ennoblements.event.refreshed")
	require.NoError(t, err)

	payload := domain.RefreshEnnoblementsCmdPayload{
		Key:         "pl151",
		URL:         "https://pl151.plemiona.pl",
		VersionCode: "pl",
	}
	require.NoError(t, msg.NewEnnoblementPublisher(pubSub, marshaler).
		CmdRefresh(context.Background(), payload))

	receivedMsgs, _ := subscriber.BulkRead(msgs, 1, 10*time.Second)
	require.Len(t, receivedMsgs, 1)
	var ennoblementsRefreshedPayload model.EnnoblementsRefreshedEvPayload
	assert.NoError(t, marshaler.Unmarshal(receivedMsgs[0], &ennoblementsRefreshedPayload))
	assert.Equal(t, payload.Key, ennoblementsRefreshedPayload.Key)
	assert.Equal(t, payload.URL, ennoblementsRefreshedPayload.URL)
	assert.Equal(t, payload.VersionCode, ennoblementsRefreshedPayload.VersionCode)
	require.Equal(t, 1, ennoblementSvc.RefreshCallCount())
	_, serverKey, serverURL := ennoblementSvc.RefreshArgsForCall(0)
	assert.Equal(t, payload.Key, serverKey)
	assert.Equal(t, payload.URL, serverURL)
}
