package msg_test

import (
	"context"
	"testing"
	"time"

	"gitea.dwysokinski.me/twhelp/core/internal/domain"
	"gitea.dwysokinski.me/twhelp/core/internal/msg"
	"gitea.dwysokinski.me/twhelp/core/internal/msg/internal/model"
	"github.com/ThreeDotsLabs/watermill/message/subscriber"
	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestEnnoblementPublisher_CmdRefresh(t *testing.T) {
	t.Parallel()

	marshaler := msg.GobMarshaler{}
	pubSub := newPubSub(t)

	payloads := []domain.RefreshEnnoblementsCmdPayload{
		{
			Key:         "pl151",
			URL:         "https://pl151.plemiona.pl",
			VersionCode: "pl",
		},
		{
			Key:         "en113",
			URL:         "https://en113.tribalwars.net",
			VersionCode: "en",
		},
	}

	msgs, err := pubSub.Subscribe(context.Background(), "ennoblements.cmd.refresh")
	require.NoError(t, err)

	require.NoError(t, msg.NewEnnoblementPublisher(pubSub, marshaler).CmdRefresh(context.Background(), payloads...))

	receivedMsgs, _ := subscriber.BulkRead(msgs, len(payloads), time.Second)
	require.Len(t, receivedMsgs, len(payloads))
	for _, m := range receivedMsgs {
		var received model.RefreshEnnoblementsCmdPayload
		assert.NoError(t, marshaler.Unmarshal(m, &received))

		found := false
		for _, payload := range payloads {
			if payload.VersionCode == received.VersionCode &&
				payload.Key == received.Key &&
				payload.URL == received.URL {
				found = true
				break
			}
		}
		assert.True(t, found)
	}
}
